--Dwarf Planets: by Nathaniel Sherry

function LevelSetup()


	SetBackdropColour(0, 0, 0)
	
	Globals.Agents.MaxSpeed=800
	Globals.Agents.MinSpeed=200

	Globals.Asteroids.MaxTrees=4
	Globals.Asteroids.MinRadius=95
    Globals.Asteroids.MaxRadius=725
    Globals.Asteroids.RadiusPowerRule=1.5
	Globals.Asteroids.SizeFromEnergy=150
	Globals.Asteroids.SizeFromStrength=150
	Globals.Asteroids.SizeFromSpeed=150
	Globals.Asteroids.MinCoreHealth=75
	Globals.Asteroids.MaxCoreHealth=450
	Globals.Asteroids.CoreHealthPower=3.0
    Globals.Asteroids.MinSendDistance=1000
    Globals.Asteroids.MaxSendDistance=2500
    Globals.Asteroids.SendPowerRule=1.4
	
	Globals.AI.MinAttackTime=(5)
	Globals.AI.MaxAttackTime=(20)
	Globals.AI.MinGatherTime=(10)
	Globals.AI.MaxGatherTime=(20)
	Globals.AI.MinScoutTime=(1)
	Globals.AI.MaxScoutTime=(5)
	Globals.AI.GraceTimer=(0)
	Globals.AI.BattleSavvy=(0.75)
	Globals.AI.RecognisableThreatProportion=(0.7)
	Globals.AI.AidEnthusiasm=(0.5)
	Globals.AI.ExpandBlind=(0.0)
	
	
	
	

	Globals.G.EnemyFactionsMin=(1)
	Globals.G.EnemyFactionsMax=(1)
	Globals.G.MinAsteroidSeparation=675
	Globals.G.MaxAsteroidNeighbourDist=2750
	Globals.G.StartingSeedlings=(60)
	Globals.G.GreysProbability=0.2
	Globals.G.GreysMin=10
	Globals.G.GreysMax=20
	
		
	Globals.Structures.FlowerProbability=(0.3)


	--distance between dwarf planets
	dwarfDistance = 10000

	--create this many dwarf systems
	dwarfsystems = CreateSystemSet(10)
	
	--create this many new players
	playersSetup(dwarfsystems, 6)
	
	SetCameraZoomNow(8)
	
end


function checkWinner()

	local winner = dwarfsystems[1].Owner
	for i=2,size(dwarfsystems) do
		if not(dwarfsystems[i].Owner == winner) then return 0 end
	end
	
	return winner

end

function playersSetup(systems, players)

	local iEnd = math.min(size(systems)-1, players)
	local pn = nil
	
	for i=1,iEnd do
	
		pn = systems[i+1]
		pn = GetAsteroid(pn.ID+1)
		pn.owner = i
		pn:AddSeedlings(60)
		
		
		pn:AddTrees(2, 1)
	
	end

end



function LevelLogic()


	Pause()
	MessageBox("Dwarf Planets\n\nAt the far reaches of the solar system, you will find many Dwarf Planets. Dwarf Planets are large enough to pull nearby asteroids into orbit, and large enough for us to travel between them.\n\nCapture all of the Dwarf Planets.")
	WaitDialog()
	Unpause()
			

	while GameRunning() do
		
		local winner = checkWinner()
		if winner == 1 then
		
			Pause()
			MessageBox("You control all of the Dwarf Planets.")
			WaitDialog()
			Unpause()
	
			Quit(true)
		elseif winner > 1 or GetEmpire(1).defeated then
		
			Pause()
			MessageBox("You have failed to capture the Dwarf Planets.")
			WaitDialog()
			Unpause()
		
			Quit(false)
		end
	
		coroutine.yield()
	end
end


function CreateSystem(x, y)

	local st, sp, en = math.random(7, 10) / 10.0, math.random(7, 10) / 10.0, math.random(7, 10) / 10.0
	
	local a = AddAsteroidWithAttribs(x, y, st, sp, en)
	local radius = math.random(1000, 1100)
	local dist = math.random(dwarfDistance -2000, dwarfDistance - 1000)
	a:setRadius(radius)
	a.SendDistance = dist
	
	local orbitals = math.random(5, 10)
	
	AddAsteroidRing(orbitals, x, y, 2250, 75)
	
	a.TreeCap = math.random(9, 12)
	a.SpawnCap = 200
	a.SeedlingCap = 1000
	a.Moveable = false
	
	return a

end

function CreateSystemSet(systems)

	systems = systems - 1

	local prevLocs = {}
	prevLocs[1] = {0, 0}
	local s = CreateSystem(0, 0)

	local createdSystems = 0

	local prevLocIndex = 1
	local prevLoc = {0, 0}
	local px, py = 0, 0
	
	local newSystem = nil
	local systemsList = {}
	systemsList[1] = s

	while (createdSystems < systems) do
		
		prevLocIndex = math.random(1, size(prevLocs))
		prevLoc = prevLocs[prevLocIndex]
		px, py = createPointAtDistance(prevLoc[1], prevLoc[2], dwarfDistance)
		
		if validPlacement(prevLocs, px, py, dwarfDistance) then 
			
			newSystem = CreateSystem(px, py)
			prevLocs[size(prevLocs)+1] = {px, py}
			createdSystems = createdSystems + 1
			
			systemsList[size(systemsList)+1] = newSystem
			
		end
			
	end
	
	return systemsList
	

end

function createPointAtDistance(cx, cy, d)
	
	local px = math.random(0, 1)
	local py = math.random(0, 1)

	local d2 = math.pow(d, 2)

	local x2 = math.random(d2)
	local x = math.sqrt(x2)
	
	local y2 = math.pow(d, 2) - x2
	local y = math.sqrt(y2)
	
	if px == 0 then x = -x end
	if py == 0 then y = -y end
	
	return cx+x, cy+y

end

-- gets the distance between two points
function distanceFromPoint(x1, y1, x2, y2)

	local x = x2 - x1
	local y = y2 - y1
	
	local d = math.sqrt(  math.pow(x,2) + math.pow(y,2)  )
	
	return d

end

function validPlacement(previousLocations, x, y, minDistance)

	for i=1,(size(previousLocations)) do
	
		local prevLoc = previousLocations[i]
		local d = distanceFromPoint(x, y, prevLoc[1], prevLoc[2])
		if d < minDistance then return false end
	
	end
	
	return true

end

function size(list)
	return table.getn(list)
end
